#!/usr/bin/env bash

# Usage ./upgrade_cse.sh kubeconfig-absolute-file-path [image_registry]


extract() {
  # Usage extract [tkg|vmware-cloud-director] [container-name:version] [destination folder]
  mkdir -p /tmp/container-fs
  mkdir -p $3
  docker pull ${image_registry}/$1/$2
  docker create --name "airgapped_container" ${image_registry}/$1/$2
  docker export "airgapped_container" | tar -C /tmp/container-fs -xvf -
  cp /tmp/container-fs/src/artifact/* $3
  docker rm "airgapped_container"
  rm -rf /tmp/container-fs
}

remove_folder() {
  # Usage remove_folder [folder path]
  if [[ -n "$1" ]];
  then
    echo "Removing folder: $1"
    rm -rf $1
    echo "$1 removed"
  else
    echo "Folder path was not set, and therefore cannot be removed"
    exit 1
  fi
}

if [[ "$#" < 1 ]];
then
	echo "Usage: $0 kubeconfig-absolute-file-path [image_registry]"
	exit 1
fi

if [[ ! -f $1 ]];
then
  echo "Error: cluster kubeconfig file: $1 not found"
  exit 1
fi

if [[ ! -n "$(command -v docker)" ]]
then
  echo "Error: docker command does not exist, please install docker on the system by referring to the README.md"
  exit 1
fi

if [[ ! -n "$(command -v kubectl)" ]]
then
  echo "Error: kubectl command does not exist, please install kubectl on the system by referring to the README.md"
  exit 1
fi

if [[ ! -n "$HOME" ]]
then
  echo "Info: \$HOME environment variable is empty or does not exist. Setting \$HOME to $(echo ~)"
  export HOME=$(echo ~)
fi

set -e

# Set the versions of the target components
# NOTE: capvcd version requires prefix "v" before version, when creating folder structure, clustercrtl.yaml to pass clusterctl upgrade
# CAPVCD version v1.2.0 is built with cluster-api v1.4.0, so it is expected to update CAPI components to v1.4.0.
capi_version="v1.5.4"
kubeadm_bootstrap_version="v1.5.4"
kubeadm_control_plane_version="v1.5.4"
clusterctl_version="v1.5.4"
cert_manager_version="v1.13.2"
# TODO: update image tags of CAPVCD to release version before GA
capvcd_version="v1.3.0"

image_registry=${2:-projects.registry.vmware.com}
base_project="vmware-cloud-director"

# TODO: update image tags of RDE Projector, CPI and CSI images to release versions before GA
rde_projector_image_name="vcd-rde-projector"
rde_projector_version="0.7.1"
cpi_image_name="cloud-provider-for-cloud-director"
cpi_version="1.6.0"
csi_image_name="cloud-director-named-disk-csi-driver"
csi_version="1.6.0"

# Create folder structure
CLUSTER_PACKAGES_FOLDER="${HOME}/cluster-upgrade-packages"

mkdir -p "${HOME}/.cluster-api"

# There is no need to create ${CLUSTER_PACKAGES_FOLDER} folder on it's own, as the next few lines will create that structure as -p flag works for the entire folder path.
mkdir -p ${CLUSTER_PACKAGES_FOLDER}/infrastructure-vcd/${capvcd_version}
mkdir -p ${CLUSTER_PACKAGES_FOLDER}/cluster-api/${capi_version}
mkdir -p ${CLUSTER_PACKAGES_FOLDER}/cert-manager/${cert_manager_version}
mkdir -p ${CLUSTER_PACKAGES_FOLDER}/bootstrap-kubeadm/${kubeadm_bootstrap_version}
mkdir -p ${CLUSTER_PACKAGES_FOLDER}/control-plane-kubeadm/${kubeadm_control_plane_version}


#clusterctl install
extract vmware-cloud-director clusterctl-airgapped:${clusterctl_version} ${CLUSTER_PACKAGES_FOLDER}/clusterctl_artifact
chmod +x ${CLUSTER_PACKAGES_FOLDER}/clusterctl_artifact/clusterctl

#capvcd manifest
extract vmware-cloud-director capvcd-manifest-airgapped:${capvcd_version} ${CLUSTER_PACKAGES_FOLDER}/capvcd_artifact
cp ${CLUSTER_PACKAGES_FOLDER}/capvcd_artifact/infrastructure-components.yaml ${CLUSTER_PACKAGES_FOLDER}/infrastructure-vcd/${capvcd_version}
cp ${CLUSTER_PACKAGES_FOLDER}/capvcd_artifact/metadata.yaml ${CLUSTER_PACKAGES_FOLDER}/infrastructure-vcd/${capvcd_version}


#cluster-api components
extract vmware-cloud-director capi-airgapped:${capi_version} ${CLUSTER_PACKAGES_FOLDER}/capi_artifact

#copy bootstrap provider components
cp ${CLUSTER_PACKAGES_FOLDER}/capi_artifact/bootstrap-components.yaml ${CLUSTER_PACKAGES_FOLDER}/bootstrap-kubeadm/${kubeadm_bootstrap_version}
cp ${CLUSTER_PACKAGES_FOLDER}/capi_artifact/metadata.yaml ${CLUSTER_PACKAGES_FOLDER}/bootstrap-kubeadm/${kubeadm_bootstrap_version}

#copy core-capi components
cp ${CLUSTER_PACKAGES_FOLDER}/capi_artifact/core-components.yaml ${CLUSTER_PACKAGES_FOLDER}/cluster-api/${capi_version}
cp ${CLUSTER_PACKAGES_FOLDER}/capi_artifact/metadata.yaml ${CLUSTER_PACKAGES_FOLDER}/cluster-api/${capi_version}

#copy kubeadm control plane components
cp ${CLUSTER_PACKAGES_FOLDER}/capi_artifact/control-plane-components.yaml ${CLUSTER_PACKAGES_FOLDER}/control-plane-kubeadm/${kubeadm_control_plane_version}
cp ${CLUSTER_PACKAGES_FOLDER}/capi_artifact/metadata.yaml ${CLUSTER_PACKAGES_FOLDER}/control-plane-kubeadm/${kubeadm_control_plane_version}

#extract cert manager
extract vmware-cloud-director cert-manager-airgapped:${cert_manager_version} ${CLUSTER_PACKAGES_FOLDER}/cert_manager_artifact
cp ${CLUSTER_PACKAGES_FOLDER}/cert_manager_artifact/cert-manager.yaml ${CLUSTER_PACKAGES_FOLDER}/cert-manager/${cert_manager_version}

#extract csi manifest
extract vmware-cloud-director csi-crs-airgapped:${csi_version} ${CLUSTER_PACKAGES_FOLDER}/csi-crs-airgapped

#extract cpi manifest
extract vmware-cloud-director cpi-crs-airgapped:${cpi_version} ${CLUSTER_PACKAGES_FOLDER}/cpi-crs-airgapped

#extract projector manifest
extract vmware-cloud-director rde-projector-airgapped:${rde_projector_version} ${CLUSTER_PACKAGES_FOLDER}/rde-projector-manifest

# Generate clusterctl.yaml
cat << EOF > ${HOME}/.cluster-api/clusterctl.yaml
providers:
  - name: "vcd"
    url: "${CLUSTER_PACKAGES_FOLDER}/infrastructure-vcd/${capvcd_version}/infrastructure-components.yaml"
    type: "InfrastructureProvider"
  - name: "cluster-api"
    url: "${CLUSTER_PACKAGES_FOLDER}/cluster-api/${capi_version}/core-components.yaml"
    type: "CoreProvider"
  - name: "kubeadm"
    url: "${CLUSTER_PACKAGES_FOLDER}/bootstrap-kubeadm/${kubeadm_bootstrap_version}/bootstrap-components.yaml"
    type: "BootstrapProvider"
  - name: "kubeadm"
    url: "${CLUSTER_PACKAGES_FOLDER}/control-plane-kubeadm/${kubeadm_control_plane_version}/control-plane-components.yaml"
    type: "ControlPlaneProvider"
cert-manager:
  url: "${CLUSTER_PACKAGES_FOLDER}/cert-manager/${cert_manager_version}/cert-manager.yaml"
  version: "${cert_manager_version}"
overridesFolder: "${HOME}"
EXP_CLUSTER_RESOURCE_SET: "true"
images:
  all:
    repository: "${image_registry}/${base_project}/cluster-api"
  infrastructure-vcd:
    repository: "${image_registry}/${base_project}"
  cert-manager:
    repository: "${image_registry}/${base_project}/jetstack"
EOF

# Execute upgrade
# NOTE: clusterctl upgrade expects prefix "v" before the version to successfully execute
export KUBECONFIG=$1
${CLUSTER_PACKAGES_FOLDER}/clusterctl_artifact/clusterctl upgrade apply \
  -i capvcd-system/vcd:${capvcd_version} \
  --core cluster-api:${capi_version} \
  --bootstrap kubeadm:${kubeadm_bootstrap_version} \
  --control-plane kubeadm:${kubeadm_control_plane_version} \
  --config "${HOME}/.cluster-api/clusterctl.yaml" --v=10

# Update CPI, CSI, Projector images

# For CSI, from 1.4.0 onwards, the controller has been changed to deployment from statefulsets. Hence, remove existing stateful set and create CSI as deployment
# If CSI 1.4.0 is already present, the existing CSI controller deployment will be replaced
sed "s~__REGISTRY__~$image_registry/$base_project~g" ${CLUSTER_PACKAGES_FOLDER}/csi-crs-airgapped/csi-controller-crs-airgap.yaml.template > ${CLUSTER_PACKAGES_FOLDER}/csi-crs-airgapped/csi-controller-crs-airgap.yaml
kubectl replace --force -f ${CLUSTER_PACKAGES_FOLDER}/csi-crs-airgapped/csi-controller-crs-airgap.yaml

sed "s~__REGISTRY__~$image_registry/$base_project~g" ${CLUSTER_PACKAGES_FOLDER}/csi-crs-airgapped/csi-node-crs-airgap.yaml.template > ${CLUSTER_PACKAGES_FOLDER}/csi-crs-airgapped/csi-node-crs-airgap.yaml
kubectl replace --force -f ${CLUSTER_PACKAGES_FOLDER}/csi-crs-airgapped/csi-node-crs-airgap.yaml

# Adding `|| true` to prevent script erroring out if Kubernetes cannot find CSI StatefulSet to delete
# This would occur if the script was ran once and had already deleted the resource, or if the CSI version being used does not have the StatefulSet
kubectl delete statefulset -n kube-system csi-vcd-controllerplugin || true

# For CPI, we will need to re-apply the updated CRS manifests as K8s v1.24+ toleration has changed to control-plane only, which previous manifests do not have.
sed "s~__REGISTRY__~$image_registry/$base_project~g" ${CLUSTER_PACKAGES_FOLDER}/cpi-crs-airgapped/cloud-director-ccm-crs-airgap.yaml.template > ${CLUSTER_PACKAGES_FOLDER}/cpi-crs-airgapped/cloud-director-ccm-crs-airgap.yaml
kubectl replace --force -f ${CLUSTER_PACKAGES_FOLDER}/cpi-crs-airgapped/cloud-director-ccm-crs-airgap.yaml

# --- Projector Upgrade Begins ---
# For Projector, we will need to re-apply the updated CRS manifests as updated Projector will have mount certificates from the Nodes.
sed "s~__REGISTRY__~$image_registry/$base_project~g" ${CLUSTER_PACKAGES_FOLDER}/rde-projector-manifest/projector-components.yaml.template > ${CLUSTER_PACKAGES_FOLDER}/rde-projector-manifest/projector-components.yaml
kubectl replace --force -f ${CLUSTER_PACKAGES_FOLDER}/rde-projector-manifest/projector-components.yaml

# since projector instance may get deleted, we will need to apply the projector instance file again
mkdir -p ${HOME}/.rde-projector
cluster_username=$(kubectl config current-context) # user@clusterName
cluster_name="${cluster_username##*@}" # get clusterName from after "@"

clusterid_secret_yaml=$(kubectl get secret -n kube-system vcloud-clusterid-secret -o yaml)
cluster_id_b64=$(echo "$clusterid_secret_yaml" | awk '/clusterid:/ {print $2}') # parse out cluster_id_b64 from 'clusterid: <cluster_id_b64>'
cluster_id_decoded=$(echo "$cluster_id_b64" | base64 -d)

ccm_configmap=$(kubectl describe configmap vcloud-ccm-configmap -n kube-system)
ccm_configmap_host=$(echo "$ccm_configmap" | grep 'host: ') # get 'host: <vcd_url>' from ccm_config_map
ccm_configmap_org=$(echo "$ccm_configmap" | grep 'org: ') # get 'org: <cluster_org_name>' from ccm_config_map
vcd_site=$(echo "$ccm_configmap_host" | awk '{print $2}') # parse out vcd_url from 'host: <vcd_url>'
cluster_org=$(echo "$ccm_configmap_org" | awk '{print $2}') # parse out cluster_org_name from 'org: <cluster_org_name>'

cat << EOF > ${HOME}/.rde-projector/projector_instance.yaml
apiVersion: capvcd.cloud-director.vmware.com/v1alpha1
kind: RDEProjector
metadata:
  name: ${cluster_name}
spec:
  rdeId: ${cluster_id_decoded}
  site: ${vcd_site}
  org: ${cluster_org}
  userContext:
    secretRef:
      name: capi-user-credentials
      namespace: ${cluster_name}-ns
EOF

kubectl apply -f ${HOME}/.rde-projector/projector_instance.yaml
# --- Projector Upgrade Ends ---

echo "Upgrade of cluster-api, capvcd, cpi, csi, rde-projector components completed successfully.....!!!"

echo "Cleaning up downloaded folders from cluster upgrade script..."
remove_folder ${CLUSTER_PACKAGES_FOLDER}
remove_folder ${HOME}/.cluster-api
remove_folder ${HOME}/.rde-projector

echo "Removed cluster packages and cluster-api folders successfully"
exit 0
